//! Game client state handling

use crate::{
    assets::Assets,
    graphics::{
        entities::{Coordinates, NodeRndr},
        Renderer,
    },
    input::InputHandle,
    viewport::Viewport,
    GameSettings,
};
use ggez::{event::EventHandler, graphics, Context, GameResult};
use rst_core::data::{Color, Level, Node, Owner, Player, Upgrade};
use std::sync::Arc;

pub struct ClientState {
    assets: Assets,
    settings: GameSettings,
    input: InputHandle,
    vp: Viewport,

    // Game state
    node: NodeRndr,
}

impl ClientState {
    pub fn new(settings: GameSettings, assets: Assets) -> Self {
        Self {
            assets,
            settings,
            vp: Viewport::new(),
            input: InputHandle::new(),
            node: NodeRndr {
                loc: Coordinates(512.0, 512.0),
                inner: Arc::new(Node {
                    id: 0,
                    health: 100.into(),
                    max_health: 100.into(),
                    owner: Owner::Player(Player {
                        id: 0,
                        name: "kookie".into(),
                        color: Color::blue(),
                        money: 1000.into(),
                    }),
                    type_: Upgrade::Relay(Level::One),
                    links: 0,
                    link_states: vec![],
                    buffer: vec![],
                }),
            },
        }
    }

    pub fn viewport(&mut self) -> &mut Viewport {
        &mut self.vp
    }
    
    pub fn assets(&self) -> &Assets {
        &self.assets
    }
}

impl EventHandler for ClientState {
    fn update(&mut self, ctx: &mut Context) -> GameResult<()> {
        // TODO: get simulation updates

        // Get new input state
        self.input.update(ctx)?;

        // Apply inputs to viewpoirt
        self.vp.apply(ctx, &self.input)?;

        // Update viewport
        self.vp.update(ctx)?;
        Ok(())
    }

    fn draw(&mut self, ctx: &mut Context) -> GameResult<()> {
        graphics::clear(ctx, graphics::Color::from_rgb(15, 15, 15));

        // Render the node
        self.node.draw(&self, ctx).unwrap();

        graphics::present(ctx)
    }
}
